#    Kye - classic puzzle game
#    Copyright (C) 2006, 2007 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from os import environ
from os.path import join, basename

class KyeDefaults:
    known_header = "[Known Levels]"
    settings_header = "[Settings]"
    
    def __init__(self):
        # Path to the config file.
        self.__cf = join(environ['HOME'], ".kye.config")
        
        # Initialise.
        self.__count = 0
        self.__known = {}
        self.__orderfiles = {}
        self.__path = {}
        self.settings = {}
        
        # Try reading the config file.
        try:
            s = open(self.__cf)
            while 1:
                # Read header lines until done.
                l = s.readline().strip()
                if l == "":
                    break
                elif l == KyeDefaults.known_header:
                    # Inside the [known levels] block; keep reading until we get to a blank line.
                    while 1:
                        l = s.readline().strip()
                        if l == "":
                            break
                        
                        # Format here is filename<TAB>known_level<TAB>known_level
                        a = l.split("\t")
                        path = a.pop(0)
                        n = basename(path)
                        self.__known[n] = a
                        self.__path[n] = path
                        
                        # Order in the config file is the recently-used order.
                        self.__orderfiles[n] = self.__count
                        self.__count = self.__count + 1
                        
                elif l == KyeDefaults.settings_header:
                    # Read into settings hash until we get to a blank line.
                    while 1:
                        l = s.readline().strip()
                        if l == "":
                            break
                        a = l.split("\t")
                        if a[0] == "Size":
                            self.settings[a[0]] = a[1]
        
        except IOError, e:
            0

    def get_known_levels(self, path):
        """Get all known level names for the given filename."""
        fname = basename(path)
        if fname in self.__known:
            return self.__known[fname]
        return []

    def add_known(self, path, level_name):
        """For this kye file, we now know this level name."""
        # Index by just the filename
        fname = basename(path)
        self.__path[fname] = path
        
        # Remember that this is the most recently loaded level
        self.__orderfiles[fname] = self.__count
        self.__count = self.__count + 1

        # Add this level to the known list for this file (adding the file to the dictionary if new)
        if fname not in self.__known:
            self.__known[fname] = []
        if level_name not in self.__known[fname]:
            self.__known[fname].append(level_name)

    def get_recent(self):
        """Returns paths to the five most recently loaded .kye files."""
        a = self.__known.keys()
        a.sort(lambda x,y: cmp(self.__orderfiles[x], self.__orderfiles[y]))
        return [self.__path[n] for n in a[-5:]]

    def save(self):
        """Try to save the configuration back to the config file."""
        try:
            s = open(self.__cf, "w")

            # Known levels etc
            s.write(KyeDefaults.known_header+"\n")
            a = self.__known.keys()
            a.sort(lambda x,y: cmp(self.__orderfiles[x], self.__orderfiles[y]))
            for k in a:
                s.write("\t".join([self.__path[k],"\t".join(self.__known[k])])+"\n")
            s.write("\n")
        
            # other settings
            s.write(KyeDefaults.settings_header+"\n")
            for setting,value in self.settings.iteritems():
                s.write("%s\t%s\n" % (setting, value))
            s.write("\n")
            
        except IOError, e:
            0
